package de.tudarmstadt.informatik.hostage.protocol;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;
import java.util.TimeZone;


/**
 * HTTP protocol
 * @author Wulf Pfeiffer
 */
public class HTTP implements Protocol<String> {
	
	
	public int getPort() {
		return 80;
	}

	
	public TALK_FIRST whoTalksFirst() {
		return TALK_FIRST.CLIENT;
	}
	
	
	public List<String> processMessage(String message) {
		List<String> response = new ArrayList<String>();
		request = message + request;

		if(!message.contains(version)){
			response.add(buildPacket(c505, ""));
		} else if(message.contains(get)) {
			response.add(buildPacket(c200, get));
		} else if(message.contains(head)) {
			response.add(buildPacket(c200, head));
		} else if(message.contains(trace)){
			response.add(buildPacket(c200, trace));
		} else if(message.contains(options)){
			response.add(buildPacket(c400, options));
		} else if(message.contains(post)){
			response.add(buildPacket(c200, post));
		} else if(message.contains(put)){
			response.add(buildPacket(c400, put));
		} else if(message.contains(delete)){
			response.add(buildPacket(c200, delete));
		} else if(message.contains(connect)){
			response.add(buildPacket(c400, connect));
		} else {
			response.add(buildPacket(c400, ""));
		}
		return response;
	}

	
	public boolean isClosed() {
		return true;
	}

	
	public boolean isSecure() {
		return false;
	}

	
	public Class<String> getType() {
		return String.class;
	}

	
	public String toString() {
		return "HTTP";
	}

	/**
	 * Builds a html response that can be sent
	 * @param code response code that was determined
	 * @param type request type that was sent by the client
	 * @return the html response
	 */
	private String buildPacket(String code, String type) {
		String doc = "";
		if(type.equals(get)) doc = htmlDoc;
		else if(type.equals(head) || type.equals(delete)) doc = "";
		else if(type.equals(trace)) doc = request;
		else doc = errorHtmlPrefix + " " + code + errorHtmlSuffix;

		return version + " " + code + headerPrefix + doc.length() + headerSuffix + doc;
	}
	
	/**
	 * Get the current time in html header format.
	 * @return the formatted server time.
	 */
	private static String getServerTime() {
	    Calendar calendar = Calendar.getInstance();
	    SimpleDateFormat dateFormat = new SimpleDateFormat(
	        "EEE, dd MMM yyyy HH:mm:ss z", Locale.US);
	    dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
	    return dateFormat.format(calendar.getTime());
	}

	private static final String serverVersion = ProtocolSettings.getHttpVersion();
	//html header pre and suffix
	private static final String headerPrefix =				
			"Date: " + getServerTime() + "\r\n" +
			"Server: " + serverVersion + " \r\n" +
			"Vary: Accept-Encoding\r\n" +
			"Content-Length: ";
	private String headerSuffix =
			"\r\n" +	
			"Keep-Alive: timeout=5, max=100\r\n" +
			"Connection: Keep-Alive\r\n" +
			"Content-Type: text/html\r\n" +
			"\r\n";
	//html website
	private String htmlDoc = 
			"<!doctype html>\n" +
			"<html lang=\"en\">\n" +
			"<head>\n" +
			"<meta charset=\"UTF-8\">\n" +
			"<title>" + ProtocolSettings.getHttpQotd() + "</title>\n" +
			"<body>" + ProtocolSettings.getHttpQotd() + "</body>\n" +
			"</head>\n" +
			"</html>";
	//html error pre and suffix
	private String errorHtmlPrefix =
			"<!doctype html>\n" +
			"<html lang=\"en\">\n" +
			"<head>\n" +
			"<meta charset=\"UTF-8\">\n" +
			"<title>";
	private String errorHtmlSuffix =
			"</title>\n" +
			"</head>\n" +
			"</html>";
	
	/** Whole request that was sent by the client */
	private String request	= "";
	private String version	= "HTTP/1.1";
	//request codes
	private String options 	= "OPTIONS";
	private String get 		= "GET";
	private String head 	= "HEAD";
	private String post		= "POST";
	private String put		= "PUT";
	private String delete	= "DELETE";
	private String trace	= "TRACE";
	private String connect	= "CONNECT";
	private String c200 	= "200 OK\r\n";
	private String c400 	= "400 Bad Request\r\n";
	private String c505 	= "505 HTTP Version not supported\r\n";
}
